%
%  aggregation_F_K.m
%
%  Numerically calculates the production function defined over
%  total capital (domestic plus foreign) and labor,
%  for the parameterized example in Appendix 5 of F. Hayashi and 
%  Edward C. Prescott, "The Depressing Effect of Agricultural Institutions
%  in Prewar Japan" (February 2006).
%
%   The user-defined function used: foc_F_K.m -- calculates the k that
%   solves the foc.
%
%  In the program, k is the total capital stock (including foreign
%  capital), Y2 is value added accruing to the second sector (which is 
%  non-agriculture plus the foreign sector, and xi is the fraction of 
%  foreign capital in the total capital stock.  The time paths of these
%  three variables will be output to the text file "Appendix_5_data.txt", 
%  which will be used in drawing Appendix Figures 2 and 3 of Appendix 5 of
%  the paper.
%


clear all;clc;echo off;

% set parameters in the calculation
N=400;
kmin=0;  % lower end of the interval for the capital stock in sector 2
kmax=4;  % upper end


% Set the model parameters
nu=0.338;  % The power in the domestic Cobb-Douglas production function
gross_r=0.15;  % return from foreign capital
kbar=2;  % at this capital stock, foreign capital is zero
C=1;
A=gross_r/(nu*(kbar^(nu-1)));
B=gross_r*C;

% Now solve the maximization problem for each capital stock
k=zeros(N,1);
Y2=zeros(N,1);  % output
xi=zeros(N,1);  % foreign capital to total capital ratio

for i=1:N;
  k(i)=i*(kmax-kmin)/N;
  x0=0.1;
  options=optimset('Display','none','TolFun',1e-14);   % Option to display output
  xi(i) = fsolve('foc_F_K',x0,options,k(i),A,B,C,nu);
  Y2(i)=B*xi(i)*k(i)/(C+xi(i))+A*((1-xi(i))^nu)*(k(i)^nu);
end;
subplot(1,2,1)
  plot(k(1:N)',xi(1:N),'--rp','MarkerSize',4);
  xlabel('k');
  ylabel('xi');
subplot(1,2,2)
  plot(log(k(1:N)),log(Y2(1:N)),'--bp','MarkerSize',3)';
  title('Reduced-form production funciton');
  xlabel('log k');
  ylabel('log Y2');

  (log(Y2(300))-log(Y2(100)))/(log(k(300)-log(k(100))))
  
table1=[k Y2 xi];
dlmwrite('Appendix5_data.txt',table1);