%
%   frontier.m
%
%  Calculates the production frontier with and without the labor barrier
%  desplayed in Figures 11 and 12 of F. Hayashi and Edward C. Prescott,
%  "The Depressing Effect of Prewar Institutions on Prewar Japan" (February
%  2006).
%
%   The user-defined functions are: "se_and_se" and "sk_given_se".
%
%   In the program, (y1_c, y2_c) is the output vector when agriculture's 
%  employment share is constrained to be 64%, q_c is the relative price
%  under the same constraint, (y1_u, y2_u, q_u ) is the corresponding 
%  triple without the constraint.  These six variables will be otuput to
%  "frontier.txt", which in turn will be imported to Sheet "frontier" of
%  the Excel file "prewar_data.xls".
%

clear all;clc;echo off;
format short;


global alpha1 alpha2 theta1 theta2 eta phi;
    % alpha1: material input share in sector 1
    % alpha2: material input share in sector 2
    % theta1: capital share in sector 1
    % theta2: capital share in sector 2
    % eta: labor share in sector 1
    % phi: cost of financial intermediation for sector 2 capital


npoints=200;
sk_start=0.3;
se_start=0.3;
K_N=xlsread('prewar_dataset','parameters','l2');
sebar=xlsread('prewar_dataset','parameters','l3');
X_Q=xlsread('prewar_dataset','parameters','l4');
X_Y=xlsread('prewar_dataset','parameters','l5');
kk=K_N/X_Y;
    % kk: detrended capital stock
    % sk_start: starting value for sector 1's share of capital stock
    % se_start: starting value for sector 1's share of labor
    % sebar: lower bound for se


% Calibrated parameter values
x=xlsread('prewar_dataset','parameters','b2:b11');
theta1=x(4); % capital contribution in sector 1
eta=x(5); % labor contribution in sector 1
alpha1=x(6); % intermediate input contribution in sector 1
alpha2=x(7); % intermediate input contribution in sector 2
theta2=x(8);  % capital share in sector 2
phi=x(10); % cost of intermediation
table1=[theta1 eta alpha1 alpha2 theta2 phi];
format short;
disp('    theta1    eta      alpha1     alpha2     theta2      phi');
disp(table1);


% gross up by alpha
theta1=theta1/(1-alpha1);
eta=eta/(1-alpha1);


%
% Start iteration   
%
% unconstrained frontier
disp('---------------------------------------------------------------');
disp('       i       qq        sk        se         y1        y2');
y1_u=zeros(npoints+1,1);
y2_u=zeros(npoints+1,1);
q_u=zeros(npoints+1,1);
qq_min=0.08;
qq_max=1.373;
for i=1:npoints+1;
    qq=qq_min+(((i-1)/npoints)^0.5)*(qq_max-qq_min);    % qq: detrended relative price
    q_u(i)=qq;
    x0=[sk_start;se_start];   % starting values for first iteration
    options=optimset('Display','none','TolFun',1e-14);   % Option to display output
    x = fsolve('sk_and_se',x0,options,kk,qq); % calculates (sk,se) without labor barrier
    sk=x(1);
    se=x(2);    
    y1_u(i)=y1_f(sk,se,kk,qq)-alpha2*(1/qq)*y2_f(sk,se,kk,qq);
    y2_u(i)=y2_f(sk,se,kk,qq)-alpha1*qq*y1_f(sk,se,kk,qq)...
            -phi*(1-sk)*kk;
    disp([i qq sk se y1_u(i) y2_u(i)]);
    x0=x;
end;

% constrained frontier
disp('---------------------------------------------------------------');
disp('      i      qq        sk        y1        y2');
y1_c=zeros(npoints+1,1);
y2_c=zeros(npoints+1,1);
q_c=zeros(npoints+1,1);
qq_min=0.141;
qq_max=5.2;
for i=1:npoints+1;
    qq=qq_min+(((i-1)/npoints)^2.2)*(qq_max-qq_min);    % qq: detrended relative price
    q_c(i)=qq;
    se=sebar;
    x0=sk_start;  % starting value for the first iteration
    options=optimset('Display','none','TolFun',1e-14);   % Option to display output
    x=fsolve('sk_given_se',x0,options,kk,se,qq);  % calculates sk with labor barrier
    sk=x;
    y1_c(i)=y1_f(sk,se,kk,qq)-alpha2*(1/qq)*y2_f(sk,se,kk,qq);
    y2_c(i)=y2_f(sk,se,kk,qq)-alpha1*qq*y1_f(sk,se,kk,qq)...
            -phi*(1-sk)*kk;
    disp([i qq sk y1_u(i) y2_u(i)]);
    x0=x;
end;


% undo detrending
y1_c=(X_Y/X_Q)*y1_c;
y1_u=(X_Y/X_Q)*y1_u;
y2_c=X_Y*y2_c;
y2_u=X_Y*y2_u;
q_c=X_Q*q_c;
q_u=X_Q*q_u;


% report results
plot(y1_c,y2_c,'r:',y1_u,y2_u,'b:'),title('production frontier');
legend('constrained','unconstrained');
table1=[y1_c y2_c q_c y1_u y2_u q_u];
dlmwrite('frontier.txt',table1,'precision',12);