%
% prewar_closed_initial.m
% 
% Uses the shooting algorithn to solve the system of nonlinear difference 
% equations described in Appendix 3 of F. Hayashi and E. C. Prescott,
% "The Depressing Effect of Agricultural Institutions in Prewar Japan",
% (February 2006).
%   
% The model is closed in that sector 1 good is non-tradable.
%
% The program solves the model with and without the labor barrier.
%
% The program utilizes a number of user-defined funcitons.
%
%  - "trends" uses "TFP".
%  - "steady_state_closed" uses "r1_f", "r2_f", "income_ratio_f", "y1_f", "y2_f".
%  - "dynamical_system_closed" uses "constr_closed", "y1_f", "y2_f", "c1_f", "c2_f", "euler_closed"
%  - "constr_closed" uses "sk_se_and_qq", "sk_se_given_se"
%  - "sk_se_and_qq" uses "r1_f", "r2_f", "income_ratio_f", "c1_f", "y1_f"
%  - "sk_se_given_se" uses "r1_f", "r2_f", "c1_f", "y1_f"
%  - "euler_closed" uses "constr_closed", "r2_f".
%
%  In the program, kk is detrended capital stock, rr is detrended co-state
% (called lambda in the paper), sk is agriculture's share of capital stock,
% se is agriculture's share of employment, and qq is detrended relative 
% price.  The time paths of these five variables are output to the text
% file "kkrr_closed_with.txt" if the labor barrier is imposed, and the text
% file "kkrr_closed_without.txt" if not imposed.  Those text files will be
% imported into Sheet "simulations" of the Excel file "prewar_dataset.xls"
%

clear all;clc;echo off;
format short;


global alpha1 alpha2 beta theta1 theta2 eta mu1 mu2 phi;
    % alpha1: material input share in sector 1
    % alpha2: material input share in sector 2
    % beta: discounting factor
    % theta1: capital share in sector 1
    % theta2: capital share in sector 2
    % eta: labor share in sector 1
    % mu1: food share in the utility function
    % mu2: non-food share in the utility function, equals 1-mu1
    % phi: cost of financial intermediation for sector 2 capital


% Parameters for model selection
barrier=1;  % 1 if barrier out of sector 1, 0 otherwise
barrier
if barrier==1;
  delete prewar_closed_with.out;
  diary prewar_closed_with.out;
else;
  delete prewar_closed_without.out;
  diary prewar_closed_without.out;
end;    


% Parameters for iteration
N=500; % maximum length of time horizon
niter=300; % number of possible trials of the intial value
relative_horizon=60;  % the horizon from the vantage point of current starting
                      % date (start_t in the program)
tol_T=1.0e-2;  % if rr (detrended co-state) at start_t+relative_horizon changes
               % by less than this times rr_ss (steady-state value of
               % co-stage), then change the base date forward
addwidth=1.0e-2;    % add this times the SS value of co-state to the band within 
                    % which the initial shadow price at the base can vary
tol1=1.0e-2; % distance from the steady state
N
niter
tol_T
addwidth
tol1


% Calibrated parameter values
x=xlsread('prewar_dataset','parameters','b2:b11');
%x=xlsread('prewar_dataset','parameters','c2:c11');
            % 'cx10:cx19' if frac_d1=0.9, 'cy10:cy19' if frac_d1=0.5
frac_d1=x(1);  % fraction of food consumption in 1885 for minimum consumption
mu1=x(2); % weight for food in the utility function
mu2=x(3);
theta1=x(4); % capital contribution in sector 1
eta=x(5); % labor contribution in sector 1
alpha1=x(6); % intermediate input contribution in sector 1
alpha2=x(7); % intermediate input contribution in sector 2
theta2=x(8);  % capital share in sector 2
beta=x(9);  % discounting factor 
phi=x(10); % cost of intermediation
table1=[frac_d1 mu1 mu2 theta1 eta alpha1 alpha2 theta2 beta phi];
format short;
disp('   frac_d1  mu1      mu2     theta1    eta    alpha1    alpha2    theta2   beta    phi');
disp(table1);


% gross up by alpha
theta1=theta1/(1-alpha1);
eta=eta/(1-alpha1);


% Load data on the path of exogenous variables for 1885-1940
x=xlsread('prewar_dataset','input','a10:o65');
nyrs=length(x);
year=x(:,1);
POP=x(:,2);  % poluation in millions
EMP1=x(:,3); % employment in sector 1 in millions
EMP2=x(:,4); % employment in sector 2 in millions
h1=x(:,5); % hours per week in sector 1
h2=x(:,6); % hours per week in sector 2
K1=x(:,7); % capital stock in sector 1
K2=x(:,8); % capital stock in sector 2
q=x(:,9); % deflator, sector 1 relative to sector 2
Y1=x(:,10); % value added, sector 1
Y2=x(:,11); % value added, sector 2
psi=x(:,12); % govt expenditure share
varatio=x(:,13); % gross value-added ratio, sector 1
delta=x(:,14); % time-varying depreciation rate
tau=x(:,15);  % time-varying capital income tax rate
clear x;
EMP=EMP1+EMP2;
K=K1+K2;
sebar=EMP1./EMP;


% Calculate TFP for 1885-1940
x=TFP(Y1,K1,h1,EMP1,Y2,K2,h2,EMP2,q);
TFP1=x(:,1);
TFP2=x(:,2);
clear x;


% Parameters for projectiong exogenous variables beyond sample period
x=xlsread('prewar_dataset','parameters','f2:f11');
h1_ss=x(1); % steady-state hours worked per week, sector 1
h2_ss=x(2); % steady-state hours worked per week, sector 2
delta_ss=x(3);  % capital depreciation rate
gf1_ss=x(4); % 1 plus steady-state growth rate of TFP, sector 1
gf2_ss=x(5); % 1 plus steady-state growth rate of TFP, sector 2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% growth after 1940 for sector 2
%gf2_ss=1.05;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n_E_ss=x(6); % 1 plus steady-state value of growth rate of employment
n_N_ss=x(6);  % ss value of pop growth is the same as employment's
psi_ss=x(7); % steady-state value of share of govt expenditure of sector2 output
tau_ss=x(8);  % tax rate on net capital income
Ebar=x(9);  % Lower bound for E1
c1_1885=x(10); % percapita real food (sector 1 good) consumption in 1885
table1=[h1_ss h2_ss delta_ss gf1_ss gf2_ss n_E_ss psi_ss tau_ss c1_1885];
format short;
disp('   h1_ss    h2_ss    delta_ss   gf1_ss    gf2_ss    n_E_ss    psi_ss     tau_ss    c1(1885)');
disp(table1);


% projected values from 1941 on
for t=nyrs+1:N;
   psi(t)=psi_ss;
   TFP1(t)=TFP1(t-1)*gf1_ss;
   TFP2(t)=TFP2(t-1)*gf2_ss;
   EMP(t)=EMP(t-1)*n_E_ss;
   POP(t)=POP(t-1)*n_N_ss;
   sebar(t)=Ebar/EMP(t);
   h1(t)=h1_ss;
   h2(t)=h2_ss;
   delta(t)=delta_ss;
   tau(t)=tau_ss;
end;


% Costant minimum food
d1=zeros(N,1);
for t=1:N;
    d1(t)=frac_d1*c1_1885;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constant tau
%tau_ss=mean(tau(1:nyrs));
%for t=1:N;
%    tau(t)=tau_ss;
%end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constant delta
%for t=1:N;
%    delta(t)=delta_ss;
%end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constant TFP2 growth from 1915
for t=31:N;
    TFP2(t)=TFP2(t-1)*gf2_ss;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constant psi
%for t=1:N;
%    psi(t)=psi_ss;
%end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constant EMP
%for t=2:N;
%    EMP(t)=EMP(t-1)*n_E_ss;
%    sebar(t)=Ebar/EMP(t);
%end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Constant hours
%for t=1:N;
%    h1(t)=h1_ss;
%    h2(t)=h2_ss;
%end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



% Define trends
x=trends(TFP1,TFP2,h1,h2,EMP,POP);
X_Y=x(:,1);
X_Q=x(:,2);
clear x;
gamma2_ss=X_Y(nyrs+1)/X_Y(nyrs);


% Calculate steady-state value of (sk,se,q,kk,rr)
x0=[0.03;0.09;0.84;2.68;0.98];
options=optimset('Display','none','TolFun',1e-14);   % Option to display output
x = fsolve('steady_state_closed',x0,options,gamma2_ss,n_E_ss,psi_ss,delta_ss,tau_ss);
sk_ss=x(1);
se_ss=x(2);
qq_ss=x(3);
kk_ss=x(4);
rr_ss=x(5);
k_y=kk_ss/...
    (qq_ss*(1-alpha1)*y1_f(sk_ss,se_ss,kk_ss,qq_ss)...
    +(1-alpha2)*y2_f(sk_ss,se_ss,kk_ss,qq_ss));
table1=[sk_ss se_ss qq_ss kk_ss rr_ss k_y];
format short;
disp('SS values, from five-equation closed-economy system');
disp('    sk_ss     se_ss    qq_ss       kk_ss     rr_ss      k_y');
disp(table1);


% Begin shooting algorithm
% initial values for calculation 
kk=zeros(N,1);
rr=zeros(N,1);
kk_ini=K(1)/(X_Y(1)*POP(1));  % actual detrended k in 1885
kk(1)=kk_ini;
rr_L=0.1*rr_ss; 
rr_H=1.1*rr_ss;
rr(1)=(rr_L+rr_H)/2;  
flag_c=zeros(niter,1);
rr_T=zeros(niter,1);  % records the value at the horizon for rr
kk_T=zeros(niter,1);  % records the value at the horizon for kk
start_t=1;   % the initial base date for shooting is 1
tmax=0;  % tmax is the t where the system is closest to the SS

% Now start shooting
for j=1:niter;
disp('----------------------------------------------');
format short;
table1=[j start_t rr(1)];
disp(' iteration   start_t   initial rr');
disp(table1);
  for t=start_t:N-1;
    x=...
    dynamical_system_closed(kk(t),rr(t),sk_ss,se_ss,qq_ss,...
            sebar(t),sebar(t+1),X_Q(t),X_Q(t+1),X_Y(t),X_Y(t+1),...
            POP(t),POP(t+1),psi(t),delta(t),delta(t+1),tau(t+1),...
            d1(t),d1(t+1),barrier);
    if imag(x(2))~=0;
        disp('complex rr!!!  t kk(t) kk(t+1) rr(t) are');
        table1=[t kk(t) kk(t+1) rr(t)];
        disp(table1);
        j=niter+1;
        break;
    end;
    kk(t+1)=real(x(1));
    rr(t+1)=real(x(2));
    % record (rr, kk) at the horizon (= start_t+relative_horizon)
    if t==start_t+relative_horizon;rr_T(j)=rr(t);kk_T(j)=kk(t);end;
    % check if the path is too far from the saddle path
    if barrier==1;kmax=2.3;else;kmax=2.5;end;
            % if kmax is too large, you get complex numbers for rr, kk next
            % period.
    if t>1 & (rr(t+1)<0.2*rr_ss | kk(t+1)>kmax*kk_ss);flag_c(j)=1;break;end;  % raise rr(1)
    if t>1 & (rr(t+1)>1.5*rr_ss | kk(t+1)<0.4*kk_ss);flag_c(j)=2;break;end; % reduce rr(1)
  end;
  % interim report
  format short;
  table1=[t kk(t) rr(t) kk(t+1) rr(t+1)];
  disp('    t         kk(t)     rr(t)     kk(t+1)    rr(t+1)');
  disp(table1);
  format long e;
  table1=[rr(start_t); rr_L; rr_H];
  disp('rr(start_t), rr_L, rr_H');
  disp(table1);
  if j>10;
    format short;
    table1=[flag_c(j-5) flag_c(j-4) flag_c(j-3) flag_c(j-2) flag_c(j-1) flag_c(j)];
    disp('flag_c');
    disp(table1);
  end;
  % progress report duirng iteration
  subplot(1,2,1)
  plot(kk(1:t+1),rr(1:t+1),'--bp','MarkerSize',3)';
  title('Phase Diagram');
  xlabel('kk');
  ylabel('rr');
  subplot(1,2,2)
  plot((1:1:t+1)',kk(1:t+1),'--rp','MarkerSize',4);
  xlabel('year');
  ylabel('kk');
  pause(0.1);
  % check convergence
  if t>150;
      temp1=sqrt((kk(150:t)-kk_ss).^2+(rr(150:t)-rr_ss).^2);
      temp2=min(temp1);
      if temp2<tol1;
        tmax=find(temp1==temp2)+149;  % the system is closest to SS at t=tmax
        flag_c(j)=10;
        break;
      end;
  end;
  % determine when to move the base for shooting
  % move the base if the path is above (below) the saddle path in one iteration and
  % belog (above) in the next
  if j>1 & t>start_t+relative_horizon ...
         & ((flag_c(j-1)==1 & flag_c(j)==2) | (flag_c(j-1)==2 & flag_c(j)==1))...
         & abs(rr_T(j-1)-rr_T(j))<tol_T*rr_ss;
      start_t=start_t+round(relative_horizon/2);
                % move the base forward by half relative_horizon
      % new bracket for the initial value
      rr_H=rr(start_t)+addwidth*rr_ss;
      rr_L=rr(start_t)-addwidth*rr_ss;
  end;
  if t>1;
      % if the path is above (below) the saddle path, set the upper (lower)
      % bound to the current intial value.
      if flag_c(j)==2;rr_H=min(rr_H,rr(start_t));end;
      if flag_c(j)==1;rr_L=max(rr_L,rr(start_t));end;
      rr(start_t)=(rr_H+rr_L)/2;
  end;
  kk(t+1:N)=0;
  rr(t+1:N)=0;
end;


% Calculate the path of (sk, se, qq) to go with the path of (kk, rr).
N=tmax;
kk=kk(1:N);
rr=rr(1:N);
sk=zeros(N,1);
se=zeros(N,1);
qq=zeros(N,1);
for t=1:N;
    [sk(t),se(t),qq(t),sk_u,se_u,qq_u]=...
          constr_closed(kk(t),rr(t),sk_ss,se_ss,qq_ss,...
          sebar(t),X_Q(t),X_Y(t),d1(t),barrier);
end;


% Report results
table1=[(1:1:N)' kk rr sk se qq];
format short e;
disp('  period      kk        rr        sk         se        qq');
disp(table1);
if barrier==1;dlmwrite('kkrr_closed_with.txt',table1,'precision',12);end;
if barrier==0;dlmwrite('kkrr_closed_without.txt',table1,'precision',12);end;


% Undo detrending and calculate series of interest
X_Q=X_Q(1:N);
X_Y=X_Y(1:N);
POP=POP(1:N);
d1=d1(1:N);
q_hat=qq.*X_Q;
K_hat_N=kk.*X_Y;
Y1_hat_N=(1-alpha1)*y1_f(sk,se,kk,qq).*X_Y./X_Q;
             % Y1_hat_N is value added
C1_hat_N=c1_f(X_Q,X_Y,qq,rr,d1);
Y2_hat_N=(1-alpha2)*y2_f(sk,se,kk,qq).*X_Y-phi*(1-sk).*K_hat_N;
             % Y2_hat_N is value added
C2_hat_N=c2_f(X_Q,X_Y,qq,rr);
ratio=income_ratio_f(sk,se,kk,qq);
format short e;
table1=[(1:1:N)' q_hat K_hat_N];
disp('    period    q_hat   K_hat_N');
disp(table1);
table1=[(1:1:N)' Y1_hat_N C1_hat_N Y2_hat_N C2_hat_N];
disp('    period    Y1_hat/N  C1_hat_N   Y2_hat/N  C2_hat_N');
disp(table1);
table1=[(1:1:N)' ratio X_Q X_Y];
disp('    period ratio   X_Q    X_Y');
disp(table1);


% Evaluate the objective function
u=u_f(C1_hat_N,C2_hat_N,d1);
V=0;
for t=1:N;
    V=V+(beta^(t-1))*u(t)/POP(t);
end;
format long;
disp('Disounted utility value');
disp(V);


% Plot K/Y
K_Y_hat=K_hat_N./(q_hat.*Y1_hat_N+Y2_hat_N);
y=K_Y_hat(1:nyrs);
z=K(1:nyrs)./(q_hat(1:nyrs).*Y1+Y2);
x=(1:1:length(y))';
plot(x,y,'r:',x,z,'b-'),title('K/Y');
legend('model','data');


% Geary-Khamis PPP calculation for the base year, for the data and model economies
x0=[1;1;1];
options=optimset('Display','none','TolFun',1e-14);   % Option to display output
x = fsolve('Geary_Khamis',x0,options,...
    Y1(51)/POP(51),Y2(51)/POP(51),q(51),Y1_hat_N(51),Y2_hat_N(51),q_hat(51)); % Call optimizer
table1=[x(1)];
format long;
disp('PPP of the model economy relative to actual economy');
disp(table1);
if barrier==1;dlmwrite('Geary_Khamis_closed_with.txt',table1,'precision',12);end;
if barrier==0;dlmwrite('Geary_Khamis_closed_without.txt',table1,'precision',12);end;


diary off;