%
% saddle.m
% 
% Calculates the steady state and the Jacobian for the autonomous system
% described in Appendix 3 (for the closed-economy) and Appendix 4 
% (for the open-economy) of F. Hayashi and Edward C. Prescott, 
%  "The Depressing Effect of Agricultural Institutions in Prewar Japan"
%  (February 2006).
%
%  The program utilizes a number of user-defined functions.
%
%  - "steady_state_closed" uses "r1_f", "r2_f", "income_ratio_f", "y1_f", "y2_f".
%  - "dynamical_system_closed" uses "constr_closed", "y1_f", "y2_f", "c1_f", "c2_f", "euler_closed"
%  - "constr_closed" uses "sk_se_and_qq", "sk_se_given_se"
%  - "sk_se_and_qq" uses "r1_f", "r2_f", "income_ratio_f", "c1_f", "y1_f"
%  - "sk_se_given_se" uses "r1_f", "r2_f", "c1_f", "y1_f"
%  - "euler_closed" uses "constr_closed", "r2_f".
%
%  - "steady_state_open" uses "r1_f", "r2_f", "income_ratio_f", "y1_f",  "y2_f".
%  - "dynamical_system_open" uses "constr_open", "y1_f", "y2_f", "c1_f", "c2_f"
%  - "sk_and_se" uses "r1_f", "r2_f", "income_ratio_f"
%  - "sk_given_se" uses "r1_f", "r2_f"
%
%

clear all;clc;echo off;
delete saddle.out;
diary saddle.out;


global alpha1 alpha2 beta theta1 theta2 eta mu1 mu2 phi;
    % alpha1: material input share in sector 1
    % alpha2: material input share in sector 2
    % beta: discounting factor
    % theta1: capital share in sector 1
    % theta2: capital share in sector 2
    % eta: labor share in sector 1
    % mu1: food share in the utility function
    % mu2: non-food share in the utility function, equals 1-mu1
    % phi: cost of financial intermediation for sector 2 capital

    
% Parameters for iteration
N=500; % maximum length of time horizon


% Calibrated parameter values
x=xlsread('prewar_dataset','parameters','b2:b11');
frac_d1=x(1);  % fraction of food consumption in 1885 for minimum consumption
mu1=x(2); % weight for food in the utility function
mu2=x(3);
theta1=x(4); % capital contribution in sector 1
eta=x(5); % labor contribution in sector 1
alpha1=x(6); % intermediate input contribution in sector 1
alpha2=x(7); % intermediate input contribution in sector 2
theta2=x(8);  % capital share in sector 2
beta=x(9);  % discounting factor 
phi=x(10); % cost of intermediation
table1=[frac_d1 mu1 mu2 theta1 eta alpha1 alpha2 theta2 beta phi];
format short;
disp('   frac_d1  mu1      mu2     theta1    eta    alpha1    alpha2    theta2   beta    phi');
disp(table1);


% gross up by alpha
theta1=theta1/(1-alpha1);
eta=eta/(1-alpha1);


% Load data on the path of exogenous variables for 1885-1940
x=xlsread('prewar_dataset','input','a10:o65');
nyrs=length(x);
year=x(:,1);
POP=x(:,2);  % poluation in millions
EMP1=x(:,3); % employment in sector 1 in millions
EMP2=x(:,4); % employment in sector 2 in millions
h1=x(:,5); % hours per week in sector 1
h2=x(:,6); % hours per week in sector 2
K1=x(:,7); % capital stock in sector 1
K2=x(:,8); % capital stock in sector 2
q=x(:,9); % deflator, sector 1 relative to sector 2
Y1=x(:,10); % value added, sector 1
Y2=x(:,11); % value added, sector 2
psi=x(:,12); % govt expenditure share
varatio=x(:,13); % gross value-added ratio, sector 1
delta=x(:,14); % time-varying depreciation rate
tau=x(:,15);  % time-varying capital income tax rate
clear x;
EMP=EMP1+EMP2;
K=K1+K2;
sebar=EMP1./EMP;


% Calculate TFP for 1885-1940
x=TFP(Y1,K1,h1,EMP1,Y2,K2,h2,EMP2,q);
TFP1=x(:,1);
TFP2=x(:,2);
clear x;
        

% Parameters for projectiong exogenous variables beyond sample period
x=xlsread('prewar_dataset','parameters','f2:f11');
h1_ss=x(1); % steady-state hours worked per week, sector 1
h2_ss=x(2); % steady-state hours worked per week, sector 2
delta_ss=x(3);  % capital depreciation rate
gf1_ss=x(4); % 1 plus steady-state growth rate of TFP, sector 1
gf2_ss=x(5); % 1 plus steady-state growth rate of TFP, sector 2
n_E_ss=x(6); % 1 plus steady-state value of growth rate of employment
n_N_ss=x(6);  % ss value of pop growth is the same as employment's
psi_ss=x(7); % steady-state value of share of govt expenditure of sector2 output
tau_ss=x(8);  % tax rate on net capital income
Ebar=x(9);  % Lower bound for E1
c1_1885=x(10); % percapita real food (sector 1 good) consumption in 1885
table1=[h1_ss h2_ss delta_ss gf1_ss gf2_ss n_E_ss psi_ss tau_ss c1_1885];
format short;
disp('   h1_ss    h2_ss    delta_ss   gf1_ss    gf2_ss    n_E_ss    psi_ss     tau_ss    c1(1885)');
disp(table1);


% projected values from 1941 on
for t=nyrs+1:N;
   psi(t)=psi_ss;
   TFP1(t)=TFP1(t-1)*gf1_ss;
   TFP2(t)=TFP2(t-1)*gf2_ss;
   EMP(t)=EMP(t-1)*n_E_ss;
   POP(t)=POP(t-1)*n_N_ss;
   sebar(t)=Ebar/EMP(t);
   h1(t)=h1_ss;
   h2(t)=h2_ss;
   delta(t)=delta_ss;
   tau(t)=tau_ss;
end;


% Define trends
x=trends(TFP1,TFP2,h1,h2,EMP,POP);
X_Y=x(:,1);
X_Q=x(:,2);
clear x;
gamma2_ss=X_Y(nyrs+1)/X_Y(nyrs);


disp('%%%%%%%%%%%%%%%%%%%%%% CLOSED ECONOMY %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
% Calculate steady-state value of (sk,se,q,kk,rr)
x0=[0.03;0.09;0.84;2.68;0.98];
options=optimset('Display','none','TolFun',1e-14);   % Option to display output
x = fsolve('steady_state_closed',x0,options,gamma2_ss,n_E_ss,psi_ss,delta_ss,tau_ss);
sk_ss=x(1);
se_ss=x(2);
qq_ss=x(3);
kk_ss=x(4);
rr_ss=x(5);
k_y=kk_ss/...
    (qq_ss*(1-alpha1)*y1_f(sk_ss,se_ss,kk_ss,qq_ss)...
    +(1-alpha2)*y2_f(sk_ss,se_ss,kk_ss,qq_ss));
table1=[sk_ss se_ss qq_ss kk_ss rr_ss k_y];
format short;
disp('SS values, from five-equation closed-economy system');
disp('    sk_ss     se_ss    qq_ss       kk_ss     rr_ss      k_y');
disp(table1);


% SS check 1: evaluate (kk(t+1),rr(t+1)) implied by the two-equation closed-economy system
sebar_0=0;
sebar_1=0;
X_Q_0=1;
X_Q_1=1;
X_Y_0=1;
X_Y_1=gamma2_ss*X_Y_0;
POP_0=1;
POP_1=n_E_ss*POP_0;
d1_0=0;  % minimum consumption unimportant in the SS
barrier=0;
z=...
    dynamical_system_closed(kk_ss,rr_ss,sk_ss,se_ss,qq_ss,...
            sebar_0,sebar_1,X_Q_0,X_Q_1,X_Y_0,X_Y_1,...
            POP_0,POP_1,psi_ss,delta_ss,delta_ss,tau_ss,...
            d1_0,d1_0,barrier);
disp('kk_ss and rr_ss calculated from the two-equation closed-economy dynamical system at SS');
disp(z(1:2));


% SS check 2: calculate s_K explicitly
r=(((gamma2_ss/beta)-1)/(1-tau_ss))+delta_ss;
temp=(r+phi)/((1-alpha2)*theta2);
temp=(1-psi_ss+(mu2/mu1)*alpha2)*temp-phi;
temp2=((alpha1+(mu2/mu1))*r)/((1-alpha1)*theta1);
sk_ss2=(1-delta_ss+temp-n_E_ss*gamma2_ss)/(temp+temp2);
disp('s_K explicitly calculated');
disp(sk_ss2);


% Calculate the Jacobian and its Eigenvalues
% The Jacobian in terms of (kk,rr)
h=1.0e-10;
xk=...
    dynamical_system_closed(kk_ss+h,rr_ss,sk_ss,se_ss,qq_ss,...
            sebar_0,sebar_1,X_Q_0,X_Q_1,X_Y_0,X_Y_1,...
            POP_0,POP_1,psi_ss,delta_ss,delta_ss,tau_ss,...
            d1_0,d1_0,barrier)...
    -z;
xk=xk/h;
xr=...
    dynamical_system_closed(kk_ss,rr_ss+h,sk_ss,se_ss,qq_ss,...
            sebar_0,sebar_1,X_Q_0,X_Q_1,X_Y_0,X_Y_1,...
            POP_0,POP_1,psi_ss,delta_ss,delta_ss,tau_ss,...
            d1_0,d1_0,barrier)...
    -z;
xr=xr/h;
J(1,1)=xk(1);
J(1,2)=xr(1);
J(2,1)=xk(2);
J(2,2)=xr(2);
disp('Jacobian from two-equation closed-economy dynamical system, for (kk,rr)');
disp(J);
disp('its eigenvalues');
disp(eig(J));


disp('%%%%%%%%%%%%%%%%%%%%%%%% OPEN ECONOMY %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
% Calculate steady-state value of (sk,se,kk,rr)
x0=[0.04;0.10;2.17;1.0];
qq_ss=q(nyrs)/X_Q(nyrs);   % SS value of qq is the 1940 value
options=optimset('Display','none','TolFun',1e-14);   % Option to display output
x = fsolve('steady_state_open',x0,options,gamma2_ss,n_E_ss,psi_ss,delta_ss,tau_ss,qq_ss);
sk_ss=x(1);
se_ss=x(2);
kk_ss=x(3);
rr_ss=x(4);
k_y=kk_ss/(qq_ss*(1-alpha1)*y1_f(sk_ss,se_ss,kk_ss,qq_ss)...
            +(1-alpha2)*y2_f(sk_ss,se_ss,kk_ss,qq_ss));
table1=[sk_ss se_ss kk_ss rr_ss k_y];
format short;
disp('SS values, from four-equation open-economy system');
disp('    sk_ss     se_ss    kk_ss     rr_ss      k_y');
disp(table1);


% SS check: evaluate (kk(t+1),rr(t+1)) implied by the two-equation open-economy system
d1=0;  % no minimum food consumption
sebar_0=0;
sebar_1=0;
X_Q_0=1;
X_Q_1=1;
X_Y_0=1;
d1_0=0;  % minimum consumption unimportant in the SS
X_Y_1=gamma2_ss*X_Y_0;
POP_0=1;
POP_1=n_E_ss*POP_0;
barrier=0;
z=...
    dynamical_system_open(kk_ss,rr_ss,sk_ss,se_ss,...
            sebar_0,sebar_1,X_Q_0,X_Q_1,X_Y_0,X_Y_1,qq_ss,qq_ss,...
            POP_0,POP_1,psi_ss,delta_ss,delta_ss,tau_ss,d1_0,barrier);
disp('kk_ss and rr_ss calculated from the two-equation open-economy dynamical system at SS');
disp(z(1:2));


% Calculate the Jacobian and its Eigenvalues
% The Jacobian in terms of (kk,rr)
h=1.0e-10;
xk=...
    dynamical_system_open(kk_ss+h,rr_ss,sk_ss,se_ss,...
            sebar_0,sebar_1,X_Q_0,X_Q_1,X_Y_0,X_Y_1,qq_ss,qq_ss,...
            POP_0,POP_1,psi_ss,delta_ss,delta_ss,tau_ss,...
            d1_0,barrier)...
    -z;
xk=xk/h;
xr=...
    dynamical_system_open(kk_ss,rr_ss+h,sk_ss,se_ss,...
            sebar_0,sebar_1,X_Q_0,X_Q_1,X_Y_0,X_Y_1,qq_ss,qq_ss,...
            POP_0,POP_1,psi_ss,delta_ss,delta_ss,tau_ss,...
            d1_0,barrier)...
    -z;
xr=xr/h;
J(1,1)=xk(1);
J(1,2)=xr(1);
J(2,1)=xk(2);
J(2,2)=xr(2);
disp('Jacobian from two-equation open-economy dynamical system, for (kk,rr)');
disp(J);
disp('its eigenvalues');
disp(eig(J));


diary off;